unit UCompiler;

interface

uses
  SysUtils, Generics.Collections, UAnalyzer, UTarget;

type

  { TCompiler }

  TCompiler = class
    SupportedAnalyzers: TObjectList<TAnalyzer>;
    SupportedTargets: TObjectList<TTarget>;
    Analyzer: TAnalyzer;
    Target: TTarget;
    MainSourceFile: string;
    procedure Compile;
    procedure RegisterTarget(TargetClass: TTargetClass);
    procedure RegisterAnalyzer(AnalyzerClass: TAnalyzerClass);
    constructor Create;
    destructor Destroy; override;
  end;

implementation

procedure TCompiler.Compile;
begin
  if Assigned(Analyzer) then Analyzer.Analyze
    else raise Exception.Create('Undefined analyzer');
  if Assigned(Target) then Target.Produce
    else raise Exception.Create('Undefined target');
  WriteLn('Compile');
end;

procedure TCompiler.RegisterTarget(TargetClass: TTargetClass);
begin
  SupportedTargets.Add(TargetClass.Create);
  if not Assigned(Target) then
    Target := SupportedTargets.Last;
end;

constructor TCompiler.Create;
begin
  SupportedTargets := TObjectList<TTarget>.Create;
  SupportedAnalyzers := TObjectList<TAnalyzer>.Create;
end;

destructor TCompiler.Destroy;
begin
  SupportedTargets.Free;
  SupportedAnalyzers.Free;
  inherited;
end;

procedure TCompiler.RegisterAnalyzer(AnalyzerClass: TAnalyzerClass);
begin
  SupportedAnalyzers.Add(AnalyzerClass.Create);
  if not Assigned(Analyzer) then
    Analyzer := SupportedAnalyzers.Last;
end;

end.
